@@ -38,15 +38,15 @@ emacspeak modules:
 
 """
 
-__id__ = "$Id: espeechfactory.py,v 1.8 2006/05/08 14:56:22 wwalker Exp $"
-__author__ = "$Author: wwalker $"
-__version__ = "$Revision: 1.8 $"
-__date__ = "$Date: 2006/05/08 14:56:22 $"
+__id__ = "$Id: espeechfactory.py,v 1.10 2006/06/02 21:38:46 richb Exp $"
+__author__ = "$Author: richb $"
+__version__ = "$Revision: 1.10 $"
+__date__ = "$Date: 2006/06/02 21:38:46 $"
 __copyright__ = "Copyright (c) 2005 Google Inc."
 __license__ = "LGPL"
 __all__=['Speaker']
 
-import os, sys
+import os
 
 import debug
 import speechserver
@@ -249,6 +249,14 @@ class SpeechServer(speechserver.SpeechServer):
         else:
             self._output.write("q {%s}\nd\n" %text)
 
+    def increaseSpeechPitch(self, step=0.5):
+        """Increase speech pitch."""
+        self._settings['average-pitch'] += step
+
+    def decreaseSpeechPitch(self, step=0.5):
+        """Decrease speech pitch."""
+        self._settings['average-pitch'] -= step
+
     def increaseSpeechRate(self, step=5):
         """Set speech rate."""
         self._settings['rate'] += step
