@@ -19,17 +19,27 @@
 
 """Provides the FocusTrackingPresenter for Orca."""
 
+__id__        = "$Id: focus_tracking_presenter.py,v 1.80 2006/07/24 01:32:30 wwalker Exp $"
+__version__   = "$Revision: 1.80 $"
+__date__      = "$Date: 2006/07/24 01:32:30 $"
+__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__license__   = "LGPL"
+
 import gobject
 import Queue
+import threading
+import time
 
 import atspi
+import braille
 import default
 import debug
+import input_event
 import orca
 import presentation_manager
 import settings
 import speech
-import threading
+import util
 
 from orca_i18n import _ # for gettext support
 
@@ -55,10 +65,16 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         # is what delves them out, we keep at most one listener to avoid
         # receiving the same event twice in a row.
         #
-        self.registry      = atspi.Registry()
-        self._knownScripts = {}
-        self._eventQueue   = Queue.Queue(0)
-        self._gidle_id     = 0
+        self.registry        = atspi.Registry()
+        self._knownScripts   = {}
+        self._eventQueue     = Queue.Queue(0)
+        self._gidleId        = 0
+        self._gidleLock      = threading.Lock()
+        self.noFocusTimestamp = 0.0
+
+        if settings.debugEventQueue:
+            self._enqueueEventCount = 0
+            self._dequeueEventCount = 0
 
     ########################################################################
     #                                                                      #
@@ -76,7 +92,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         if self._listenerCounts.has_key(eventType):
             self._listenerCounts[eventType] += 1
         else:
-            self.registry.registerEventListener(self._enqueueObjectEvent,
+            self.registry.registerEventListener(self._enqueueEvent,
                                                 eventType)
             self._listenerCounts[eventType] = 1
 
@@ -89,7 +105,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
         self._listenerCounts[eventType] -= 1
         if self._listenerCounts[eventType] == 0:
-            self.registry.deregisterEventListener(self._enqueueObjectEvent,
+            self.registry.deregisterEventListener(self._enqueueEvent,
                                                   eventType)
             del self._listenerCounts[eventType]
 
@@ -165,6 +181,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                                       "...found %s.py" % name)
                         break
                     except ImportError:
+                        debug.printException(debug.LEVEL_ALL)
                         debug.println(debug.LEVEL_FINEST,
                                       "...could not find %s.py" % name)
                     except:
@@ -204,6 +221,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 debug.println(debug.LEVEL_FINE,
                               "...found %s.py" % name)
             except ImportError:
+                debug.printException(debug.LEVEL_ALL)
                 debug.println(
                     debug.LEVEL_FINE,
                     "...could not find %s.py" % app.toolkitName)
@@ -228,11 +246,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         Returns an instance of a Script.
         """
 
-        # We might not know what the app is.  In this case, just defer to the
-        # default script for support.  Note the hack to check for Orca - this
-        # will occur if Orca pops up its own windows.  We work to make Orca
-        # windows work well with the default script so it will not need a
-        # custom script.
+        # We might not know what the app is.  In this case, just defer
+        # to the default script for support.
         #
         if not app:
             if not self._defaultScript:
@@ -253,24 +268,30 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         deleting any scripts as necessary.
         """
 
-        apps = []
-
-        desktop = self.registry.desktop
-        for i in range(0, desktop.childCount):
-            try:
+        # Sometimes the desktop can become unavailable.  This happens
+        # often when synaptic is used to load new modules (see the bug
+        # report http://bugzilla.gnome.org/show_bug.cgi?id=342022).
+        # So...if this happens, we'll just move along.  The next
+        # successful call to _reclaimScripts will reclaim anything we
+        # didn't reclaim this time.
+        #
+        try:
+            apps = []
+            desktop = self.registry.desktop
+            for i in range(0, desktop.childCount):
                 acc = desktop.getChildAtIndex(i)
                 app = atspi.Accessible.makeAccessible(acc)
                 if app:
                     apps.insert(0, app)
-            except:
-                debug.printException(debug.LEVEL_FINEST)
 
-        for app in self._knownScripts.keys():
-            if apps.count(app) == 0:
-                script = self._knownScripts[app]
-                self._deregisterEventListeners(script)
-                del self._knownScripts[app]
-                del script
+            for app in self._knownScripts.keys():
+                if apps.count(app) == 0:
+                    script = self._knownScripts[app]
+                    self._deregisterEventListeners(script)
+                    del self._knownScripts[app]
+                    del script
+        except:
+            debug.printException(debug.LEVEL_FINEST)
 
     ########################################################################
     #                                                                      #
@@ -279,96 +300,32 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
     #                                                                      #
     ########################################################################
 
-    def _enqueueObjectEvent(self, e):
-        """Handles all events destined for scripts.
+    def _processKeyboardEvent(self, keyboardEvent):
+        """Processes the given keyboard event based on the keybinding from the
+        currently active script.
 
         Arguments:
-        - e: an at-spi event.
+        - keyboardEvent: an instance of input_event.KeyboardEvent
         """
+        if self._activeScript:
+            try:
+                self._activeScript.processKeyboardEvent(keyboardEvent)
+            except:
+                debug.printException(debug.LEVEL_WARNING)
+                debug.printStack(debug.LEVEL_WARNING)
 
-        # Uncomment these lines if you want to see what it's like without
-        # the queue.
-        #
-        #self._processObjectEvent(atspi.Event(e))
-        #return
+    def _processBrailleEvent(self, brailleEvent):
+        """Called whenever a cursor key is pressed on the Braille display.
 
-        # We ignore defunct objects and let the atspi module take care of them
-        # for us.
-        #
-        if (e.type == "object:state-changed:defunct"):
-            return
-
-        # We also generally do not like
-        # object:property-change:accessible-parent events because they
-        # indicate something is now whacked with the hierarchy, so we
-        # just ignore them and let the atspi module take care of it for
-        # us.
-        #
-        if e.type == "object:property-change:accessible-parent":
-            return
-
-        # At this point in time, we only care when objects are removed
-        # from the desktop.
-        #
-        if (e.type == "object:children-changed:remove") \
-           and (e.source != self.registry.desktop):
-            return
-
-        # We create the event here because it will ref everything we
-        # want it to ref, thus allowing things to survive until they
-        # are processed on the gidle thread.
-        #
-        debug.println(debug.LEVEL_FINEST, "Queueing event %s" % e.type)
-        try:
-            self._eventQueue.put(atspi.Event(e))
-            if not self._gidle_id:
-                self._gidle_id = gobject.idle_add(self._dequeueObjectEvent)
-        except:
-            debug.printException(debug.LEVEL_SEVERE)
-            debug.println(debug.LEVEL_SEVERE,
-                          "Exception above while processing event: " + e.type)
-
-    def _timeout(self):
-        """Timer that will be called if we time out while trying to perform
-        an operation."""
-        debug.println(debug.LEVEL_SEVERE,
-                      "TIMEOUT: Looks like something has hung. Quitting Orca.")
-        orca.shutdown()
-
-    def _dequeueObjectEvent(self):
-        """Handles all events destined for scripts.  Called by the GTK
-        idle thread.
+        Arguments:
+        - brailleEvent: an instance of input_event.BrailleEvent
         """
-
-        event = self._eventQueue.get()
-        debug.println(debug.LEVEL_FINEST, "Dequeued event %s" % event.type)
-
-        # [[[TODO: WDW - the timer stuff is an experiment to see if
-        # we can recover from hangs.  It's only experimental, so it's
-        # commented out for now.]]]
-        #
-        #timer = threading.Timer(5.0, self._timeout)
-        #timer.start()
-
-        debug.println(debug.eventDebugLevel,
-                      "\nvvvvv PROCESS OBJECT EVENT vvvvv")
-        
-        try:
-            self._processObjectEvent(event)
-        except:
-            pass
-        
-        debug.println(debug.eventDebugLevel,
-                      "^^^^^ PROCESS OBJECT EVENT ^^^^^\n")
-
-        #timer.cancel()
-        #del timer
-
-        if self._eventQueue.empty():
-            self._gidle_id = 0
-            return False
-        else:
-            return True
+        if self._activeScript:
+            try:
+                self._activeScript.processBrailleEvent(brailleEvent)
+            except:
+                debug.printException(debug.LEVEL_WARNING)
+                debug.printStack(debug.LEVEL_WARNING)
 
     def _processObjectEvent(self, event):
         """Handles all events destined for scripts.
@@ -396,52 +353,329 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             return
 
         try:
+            # We don't want to touch a defunct object.  It's useless and it
+            # can cause hangs.
+            #
+            if event.source \
+               and event.source.state.count(atspi.Accessibility.STATE_DEFUNCT):
+                debug.println(debug.LEVEL_FINEST,
+                              "IGNORING DEFUNCT OBJECT")
+                atspi.Accessible.deleteAccessible(event.source)
+                return
+
             debug.printDetails(debug.LEVEL_FINEST, "    ", event.source)
+
         except CORBA.COMM_FAILURE:
-            debug.printException(debug.LEVEL_FINEST)
+            debug.printException(debug.LEVEL_WARNING)
             debug.println(debug.LEVEL_FINEST,
                           "COMM_FAILURE above while processing event: " \
                           + event.type)
-            atspi.Accessible.deleteAccessible(event.source)
-            return
         except CORBA.OBJECT_NOT_EXIST:
-            debug.printException(debug.LEVEL_SEVERE)
-            debug.println(debug.LEVEL_SEVERE,
+            debug.printException(debug.LEVEL_WARNING)
+            debug.println(debug.LEVEL_WARNING,
                           "OBJECT_NOT_EXIST above while processing event: " \
                           + event.type)
             atspi.Accessible.deleteAccessible(event.source)
             return
         except:
-            debug.printException(debug.LEVEL_SEVERE)
+            debug.printException(debug.LEVEL_WARNING)
             return
 
         if not event.source:
-            debug.println(debug.LEVEL_SEVERE,
+            debug.println(debug.LEVEL_WARNING,
                           "ERROR: received an event with no source.")
             return
 
+        # We can sometimes get COMM_FAILURES even if the object has not
+        # gone away.  This happens a lot with the Java access bridge.
+        # So...we will try a few times before giving up.
+        #
         # [[[TODO: WDW - might want to consider re-introducing the reload
         # feature of scripts somewhere around here.  I pulled it out as
         # part of the big refactor to make scripts object-oriented. Logged
         # as bugzilla bug 319777.]]]
         #
+        retryCount = 0
+        oldLocusOfFocus = orca.locusOfFocus
+        while retryCount <= settings.commFailureAttemptLimit:
+            try:
+                if event.type == "window:activate":
+                    # We'll let someone else decide if it's important
+                    # to stop speech or not.
+                    #speech.stop()
+                    self._activeScript = self._getScript(event.source.app)
+                    debug.println(debug.LEVEL_FINE, "ACTIVE SCRIPT: " \
+                                  + self._activeScript.name)
+                s = self._getScript(event.source.app)
+                s.processObjectEvent(event)
+                if retryCount:
+                    debug.println(debug.LEVEL_WARNING,
+                                  "  SUCCEEDED AFTER %d TRIES" % retryCount)
+                break
+            except CORBA.COMM_FAILURE:
+                debug.printException(debug.LEVEL_WARNING)
+                debug.println(debug.LEVEL_WARNING,
+                              "COMM_FAILURE above while processing: " \
+                              + event.type)
+                retryCount += 1
+                if retryCount <= settings.commFailureAttemptLimit:
+                    # We want the old locus of focus to be reset so
+                    # the proper stuff will be spoken if the locus
+                    # of focus changed during our last attempt at
+                    # handling this event.
+                    #
+                    orca.locusOfFocus = oldLocusOfFocus
+                    debug.println(debug.LEVEL_WARNING,
+                                  "  TRYING AGAIN (%d)" % retryCount)
+                    time.sleep(settings.commFailureWaitTime)
+                else:
+                    debug.println(debug.LEVEL_WARNING,
+                                  "  GIVING UP AFTER %d TRIES" \
+                                  % (retryCount - 1))
+                    atspi.Accessible.deleteAccessible(event.source)
+            except:
+                debug.printException(debug.LEVEL_WARNING)
+                break
+
+    def _enqueueEvent(self, e):
+        """Handles all events destined for scripts.
+
+        Arguments:
+        - e: an at-spi event.
+        """
+
+        # Uncomment these lines if you want to see what it's like without
+        # the queue.
+        #
+        #if isinstance(e, input_event.KeyboardEvent):
+        #    self._processKeyboardEvent(e)
+        #elif isinstance(e, input_event.BrailleEvent):
+        #    self._processBrailleEvent(e)
+        #else:
+        #    self._processObjectEvent(atspi.Event(e))
+        #return
+
+        if settings.debugEventQueue:
+            if self._enqueueEventCount:
+                debug.println(debug.LEVEL_ALL,
+                              "focus_tracking_presenter._enqueueEvent has " \
+                              "been entered before exiting (count = %d)" \
+                              % self._enqueueEventCount)
+            self._enqueueEventCount += 1
+
+        event = None
+        if isinstance(e, input_event.KeyboardEvent):
+            if e.type == atspi.Accessibility.KEY_PRESSED_EVENT:
+                debug.println(debug.LEVEL_FINEST,
+                              "----------> QUEUEING KEYPRESS '%s' (%d)"
+                              % (e.event_string, e.hw_code))
+            elif e.type == atspi.Accessibility.KEY_RELEASED_EVENT:
+                debug.println(debug.LEVEL_FINEST,
+                              "----------> QUEUEING KEYRELEASE '%s' (%d)"
+                              % (e.event_string, e.hw_code))
+            event = e
+        elif isinstance(e, input_event.BrailleEvent):
+            debug.println(debug.LEVEL_FINEST,
+                          "----------> QUEUEING BRAILLE COMMAND %d" % e.event)
+            event = e
+        else:
+            # We ignore defunct objects and let the atspi module take
+            # care of them for us.
+            #
+            if (e.type == "object:state-changed:defunct"):
+                if settings.debugEventQueue:
+                    self._enqueueEventCount -= 1
+                return
+
+            # We also generally do not like
+            # object:property-change:accessible-parent events because
+            # they indicate something is now whacked with the
+            # hierarchy, so we just ignore them and let the atspi
+            # module take care of it for us.
+            #
+            if e.type == "object:property-change:accessible-parent":
+                if settings.debugEventQueue:
+                    self._enqueueEventCount -= 1
+                return
+
+            # At this point in time, we only care when objects are
+            # removed from the desktop.
+            #
+            if (e.type == "object:children-changed:remove") \
+                and (e.source != self.registry.desktop):
+                if settings.debugEventQueue:
+                    self._enqueueEventCount -= 1
+                return
+
+            # We create the event here because it will ref everything
+            # we want it to ref, thus allowing things to survive until
+            # they are processed on the gidle thread.
+            #
+            debug.println(debug.LEVEL_FINEST,
+                          "---------> QUEUEING EVENT %s" % e.type)
+            event = atspi.Event(e)
+
+        if event:
+            if settings.debugEventQueue:
+                debug.println(debug.LEVEL_ALL,
+                              "           acquiring lock...")
+            self._gidleLock.acquire()
+            if settings.debugEventQueue:
+                debug.println(debug.LEVEL_ALL,
+                              "           ...acquired")
+            if settings.debugEventQueue:
+                debug.println(debug.LEVEL_ALL,
+                              "           calling queue.put...")
+            if settings.debugEventQueue:
+                debug.println(debug.LEVEL_ALL,
+                              "           (full=%s)" % self._eventQueue.full())
+            self._eventQueue.put(event)
+            if settings.debugEventQueue:
+                debug.println(debug.LEVEL_ALL,
+                              "           ...put complete")
+            if not self._gidleId:
+                time.sleep(0.0001) # Attempt to sidestep GIL
+                self._gidleId = gobject.idle_add(self._dequeueEvent)
+
+            if settings.debugEventQueue:
+                debug.println(debug.LEVEL_ALL,
+                              "           releasing lock...")
+            self._gidleLock.release()
+            if settings.debugEventQueue:
+                debug.println(debug.LEVEL_ALL,
+                              "           ...released")
+        if settings.debugEventQueue:
+            self._enqueueEventCount -= 1
+
+        # [[[TODO: HACK - on some hangs, we see the event queue growing,
+        # but the thread to take the events off is hung.  We might be
+        # able to 'recover' by quitting when we see this happen.]]]
+        #
+        #if self._eventQueue.qsize() > 500:
+        #    print "Looks like something has hung.  Here's the threads:"
+        #    for someThread in threading.enumerate():
+        #        print someThread.getName(), someThread.isAlive()
+        #    print "Quitting Orca."
+        #    orca.shutdown()
+
+    def _timeout(self):
+        """Timer that will be called if we time out while trying to perform
+        an operation."""
+        debug.println(debug.LEVEL_SEVERE,
+                      "TIMEOUT: Looks like something has hung. Quitting Orca.")
+        orca.shutdown()
+
+    def _dequeueEvent(self):
+        """Handles all events destined for scripts.  Called by the GTK
+        idle thread.
+        """
+
+        rerun = True
+
+        if settings.debugEventQueue:
+            debug.println(debug.LEVEL_ALL,
+                          "Entering focus_tracking_presenter._dequeueEvent" \
+                          + " %d" % self._dequeueEventCount)
+            self._dequeueEventCount += 1
+
         try:
-            if event.type == "window:activate":
-                speech.stop()
-                self._activeScript = self._getScript(event.source.app)
-                debug.println(debug.LEVEL_FINE, "ACTIVE SCRIPT: " \
-                              + self._activeScript.name)
-            s = self._getScript(event.source.app)
-            s.processObjectEvent(event)
-        except CORBA.COMM_FAILURE:
-            debug.printException(debug.LEVEL_SEVERE)
+            event = self._eventQueue.get_nowait()
+
+            if isinstance(event, input_event.KeyboardEvent):
+                if event.type == atspi.Accessibility.KEY_PRESSED_EVENT:
+                    debug.println(debug.LEVEL_FINEST,
+                                  "DEQUEUED KEYPRESS '%s' (%d) <----------" \
+                                  % (event.event_string, event.hw_code))
+                    pressRelease = "PRESS"
+                elif event.type == atspi.Accessibility.KEY_RELEASED_EVENT:
+                    debug.println(debug.LEVEL_FINEST,
+                                  "DEQUEUED KEYRELEASE '%s' (%d) <----------" \
+                                  % (event.event_string, event.hw_code))
+                    pressRelease = "RELEASE"
+                debug.println(debug.eventDebugLevel,
+                              "\nvvvvv PROCESS KEY %s EVENT %s vvvvv"\
+                              % (pressRelease, event.event_string))
+                self._processKeyboardEvent(event)
+                debug.println(debug.eventDebugLevel,
+                              "\n^^^^^ PROCESS KEY %s EVENT %s ^^^^^"\
+                              % (pressRelease, event.event_string))
+            elif isinstance(event, input_event.BrailleEvent):
+                debug.println(debug.LEVEL_FINEST,
+                              "DEQUEUED BRAILLE COMMAND %d <----------" \
+                              % event.event)
+                debug.println(debug.eventDebugLevel,
+                              "\nvvvvv PROCESS BRAILLE EVENT %d vvvvv"\
+                              % event.event)
+                self._processBrailleEvent(event)
+                debug.println(debug.eventDebugLevel,
+                              "\n^^^^^ PROCESS BRAILLE EVENT %d ^^^^^"\
+                              % event.event)
+            else:
+                debug.println(debug.LEVEL_FINEST, "DEQUEUED EVENT %s <----------" \
+                              % event.type)
+
+                # [[[TODO: WDW - the timer stuff is an experiment to see if
+                # we can recover from hangs.  It's only experimental, so it's
+                # commented out for now.]]]
+                #
+                #timer = threading.Timer(5.0, self._timeout)
+                #timer.start()
+
+                if (not debug.eventDebugFilter) \
+                    or (debug.eventDebugFilter \
+                        and debug.eventDebugFilter.match(event.type)):
+                    debug.println(debug.eventDebugLevel,
+                                  "\nvvvvv PROCESS OBJECT EVENT %s vvvvv" \
+                                  % event.type)
+                self._processObjectEvent(event)
+                if (not debug.eventDebugFilter) \
+                    or (debug.eventDebugFilter \
+                        and debug.eventDebugFilter.match(event.type)):
+                    debug.println(debug.eventDebugLevel,
+                                  "^^^^^ PROCESS OBJECT EVENT %s ^^^^^\n" \
+                                  % event.type)
+
+                #timer.cancel()
+                #del timer
+
+            # [[[TODO: HACK - it would seem logical to only do this if we
+            # discover the queue is empty, but this inroduces a hang for
+            # some reason if done inside an acquire/release block for a
+            # lock.  So...we do it here.]]]
+            #
+            noFocus = (not self._activeScript) or ((not orca.locusOfFocus) \
+                      and (self.noFocusTimestamp != orca.noFocusTimeStamp))
+
+            self._gidleLock.acquire()
+            if self._eventQueue.empty():
+                if noFocus:
+                    time.sleep(0.0001) # Attempt to sidestep GIL
+                    message = _("No focus")
+                    if settings.brailleVerbosityLevel == \
+                        settings.VERBOSITY_LEVEL_VERBOSE:
+                        braille.displayMessage(message)
+                    if settings.speechVerbosityLevel == \
+                        settings.VERBOSITY_LEVEL_VERBOSE:
+                        speech.speak(message)
+                    self.noFocusTimestamp = orca.noFocusTimeStamp
+                self._gidleId = 0
+                rerun = False # destroy and don't call again
+            self._gidleLock.release()
+        except Queue.Empty:
             debug.println(debug.LEVEL_SEVERE,
-                          "COMM_FAILURE above while processing event: " \
-                          + event.type)
-            atspi.Accessible.deleteAccessible(event.source)
+                          "focus_tracking_presenter:_dequeueEvent: " \
+                          + " the event queue is empty!")
         except:
             debug.printException(debug.LEVEL_SEVERE)
 
+        if settings.debugEventQueue:
+            self._dequeueEventCount -= 1
+            debug.println(debug.LEVEL_ALL,
+                          "Leaving focus_tracking_presenter._dequeueEvent" \
+                          + " %d" % self._dequeueEventCount)
+
+        return rerun
+
     def processKeyboardEvent(self, keyboardEvent):
         """Processes the given keyboard event based on the keybinding from the
         currently active script. This method is called synchronously from the
@@ -454,8 +688,10 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         Returns True if the event should be consumed.
         """
 
-        if self._activeScript:
-            return self._activeScript.processKeyboardEvent(keyboardEvent)
+        if self._activeScript \
+           and self._activeScript.consumesKeyboardEvent(keyboardEvent):
+            self._enqueueEvent(keyboardEvent)
+            return True
         else:
             return False
 
@@ -468,8 +704,10 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         Returns True if the command was consumed; otherwise False
         """
 
-        if self._activeScript:
-            return self._activeScript.processBrailleEvent(brailleEvent)
+        if self._activeScript \
+           and self._activeScript.consumesBrailleEvent(brailleEvent):
+            self._enqueueEvent(brailleEvent)
+            return True
         else:
             return False
 
@@ -512,13 +750,13 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         self._listenerCounts = {}
         self._knownScripts   = {}
         self._defaultScript  = None
-        self._activeScript   = None
+        self._activeScript   = self._getScript(None)
 
         self._registerEventListener("window:activate")
         self._registerEventListener("window:deactivate")
         self._registerEventListener("object:children-changed:remove")
 
-        win = orca.findActiveWindow()
+        win = util.findActiveWindow()
         if win:
             # Generate a fake window activation event so the application
             # can tell the user about itself.
@@ -529,13 +767,13 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             e.detail1  = 0
             e.detail2  = 0
             e.any_data = None
-            self._enqueueObjectEvent(e)
+            self._enqueueEvent(e)
 
     def deactivate(self):
         """Called when this presentation manager is deactivated."""
 
         for eventType in self._listenerCounts.keys():
-            self.registry.deregisterEventListener(self._enqueueObjectEvent,
+            self.registry.deregisterEventListener(self._enqueueEvent,
                                                   eventType)
         self._listenerCounts = {}
         self._knownScripts   = {}
