@@ -19,13 +19,17 @@
 
 """Provides a SpeechServer factory for gnome-speech drivers."""
 
-__id__        = "$Id: gnomespeechfactory.py,v 1.44 2006/07/09 18:13:28 wwalker Exp $"
-__version__   = "$Revision: 1.44 $"
-__date__      = "$Date: 2006/07/09 18:13:28 $"
+__id__        = "$Id: gnomespeechfactory.py,v 1.48 2006/07/18 14:15:46 wwalker Exp $"
+__version__   = "$Revision: 1.48 $"
+__date__      = "$Date: 2006/07/18 14:15:46 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
+import gobject
+import Queue
 import string
+import threading
+import time
 
 import bonobo
 
@@ -240,11 +244,14 @@ class SpeechServer(speechserver.SpeechServer):
         self.__pitchInfo  = {}
         self.__rateInfo   = {}
         self.__volumeInfo = {}
-        self.__driver = driver
+        self.__driver     = driver
         self.__driverName = driver.driverName
-        self.__iid = iid
-        self.__sayAll = None
+        self.__iid        = iid
+        self.__sayAll     = None
         self.__isSpeaking = False
+        self.__eventQueue = Queue.Queue(0)
+        self.__gidleId    = 0
+        self.__gidleLock  = threading.Lock()
 
     def __getRate(self, speaker):
         """Gets the voice-independent ACSS rate value of a voice."""
@@ -406,28 +413,28 @@ class SpeechServer(speechserver.SpeechServer):
 
         return speaker
 
-    def notify(self, type, id, offset):
-        """Called by GNOME Speech when the GNOME Speech driver generates
-        a callback.  This is for internal use only.
+    def __idleHandler(self):
+        """Called by the gidle thread when there is something to do.
+        The goal is to try to do all AT-SPI interactions on the gidle
+        thread as a means to help prevent hangs."""
+
+        # Added in the notify method below.
+        #
+        # id     = the id of the utterance we sent to gnome-speech
+        # type   = the type of progress we're getting
+        # offset = character offset into the utterance
+        #
+        (id, type, offset) = self.__eventQueue.get()
 
-        Arguments:
-        - type:   one of GNOME.Speech.speech_callback_speech_started,
-                         GNOME.Speech.speech_callback_speech_progress,
-                         GNOME.Speech.speech_callback_speech_ended
-        - id:     the id of the utterance (returned by say)
-        - offset: the character offset into the utterance (for progress)
-        """
         if self.__sayAll:
             if self.__sayAll.idForCurrentContext == id:
                 context = self.__sayAll.currentContext
                 if type == GNOME.Speech.speech_callback_speech_started:
-                    self.__isSpeaking = True
                     context.currentOffset = context.startOffset
                     self.__sayAll.progressCallback(
                         self.__sayAll.currentContext,
                         speechserver.SayAllContext.PROGRESS)
                 elif type == GNOME.Speech.speech_callback_speech_progress:
-                    self.__isSpeaking = True
                     context.currentOffset = context.startOffset + offset
                     self.__sayAll.progressCallback(
                         self.__sayAll.currentContext,
@@ -450,13 +457,45 @@ class SpeechServer(speechserver.SpeechServer):
                             self.__sayAll.currentContext,
                             speechserver.SayAllContext.COMPLETED)
                         self.__sayAll = None
-        elif type == GNOME.Speech.speech_callback_speech_started:
+
+        rerun = True
+
+        self.__gidleLock.acquire()
+        if self.__eventQueue.empty():
+            self.__gidleId = 0
+            rerun = False # destroy and don't call again
+        self.__gidleLock.release()
+
+        return rerun
+
+    def notify(self, type, id, offset):
+        """Called by GNOME Speech when the GNOME Speech driver generates
+        a callback.  This is for internal use only.
+
+        Arguments:
+        - type:   one of GNOME.Speech.speech_callback_speech_started,
+                         GNOME.Speech.speech_callback_speech_progress,
+                         GNOME.Speech.speech_callback_speech_ended
+        - id:     the id of the utterance (returned by say)
+        - offset: the character offset into the utterance (for progress)
+        """
+
+        if type == GNOME.Speech.speech_callback_speech_started:
             self.__isSpeaking = True
         elif type == GNOME.Speech.speech_callback_speech_progress:
             self.__isSpeaking = True
-        elif type == GNOME.Speech.speech_callback_speech_ended:
+        elif (type == GNOME.Speech.speech_callback_speech_ended) \
+            and (not self.__sayAll):
             self.__isSpeaking = False
 
+        if self.__sayAll:
+            self.__gidleLock.acquire()
+            self.__eventQueue.put((id, type, offset))
+            if not self.__gidleId:
+                time.sleep(0.0001) # Attempt to sidestep GIL
+                self.__gidleId = gobject.idle_add(self.__idleHandler)
+            self.__gidleLock.release()
+
     def getInfo(self):
         """Returns [driverName, serverId]
         """
