@@ -24,9 +24,9 @@
 
 """Displays a GUI for the user to set Orca preferences."""
 
-__id__        = "$Id: orca_gui_prefs.py,v 1.46 2006/07/09 19:13:22 wwalker Exp $"
-__version__   = "$Revision: 1.46 $"
-__date__      = "$Date: 2006/07/09 19:13:22 $"
+__id__        = "$Id: orca_gui_prefs.py,v 1.48 2006/07/21 19:49:03 wwalker Exp $"
+__version__   = "$Revision: 1.48 $"
+__date__      = "$Date: 2006/07/21 19:49:03 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -127,6 +127,8 @@ class orcaSetupGUI(GladeWrapper):
             debug.println(debug.LEVEL_SEVERE, _("Speech not available."))
             debug.printStack(debug.LEVEL_FINEST)
             self.prefsDict["enableSpeech"] = False
+            self.factory = None
+            self.server = None
             return
         elif len(self.workingFactories) > 1:
             i = 1
@@ -218,7 +220,7 @@ class orcaSetupGUI(GladeWrapper):
         else:
             self.allButton.set_active(True)
 
-        if prefs["speechVerbosityLevel"] == _("Brie_f"):
+        if prefs["speechVerbosityLevel"] == settings.VERBOSITY_LEVEL_BRIEF:
             self.speechBriefButton.set_active(True)
         else:
             self.speechVerboseButton.set_active(True)
@@ -238,7 +240,7 @@ class orcaSetupGUI(GladeWrapper):
         state = prefs["brailleRolenameStyle"] == \
                             settings.BRAILLE_ROLENAME_STYLE_SHORT
         self.abbrevRolenames.set_active(state)
-        if prefs["brailleVerbosityLevel"] == _("Brief"):
+        if prefs["brailleVerbosityLevel"] == settings.VERBOSITY_LEVEL_BRIEF:
             self.brailleBriefButton.set_active(True)
         else:
             self.brailleVerboseButton.set_active(True)
