@@ -19,8 +19,13 @@
 # Boston, MA 02111-1307, USA.
 
 """Provides a methods that converts the role name of an Accessible
-object into localized strings for speech and braille.
-"""
+object into localized strings for speech and braille."""
+
+__id__        = "$Id: rolenames.py,v 1.26 2006/07/13 13:46:48 wwalker Exp $"
+__version__   = "$Revision: 1.26 $"
+__date__      = "$Date: 2006/07/13 13:46:48 $"
+__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__license__   = "LGPL"
 
 import debug
 import settings
@@ -365,7 +370,7 @@ rolenames[ROLE_RADIO_BUTTON] = Rolename(ROLE_RADIO_BUTTON,
 
 rolenames[ROLE_RADIO_MENU_ITEM] = Rolename(ROLE_RADIO_MENU_ITEM,
                                            _("rdmnuitm"),
-                                           _("RadioMenuItem"),
+                                           _("RadioItem"),
                                            _("radio menu item"))
 
 rolenames[ROLE_RADIO_MENU] = Rolename(ROLE_RADIO_MENU,
