@@ -17,9 +17,18 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
+"""Custom script for Evolution."""
+
+__id__        = "$Id: Evolution.py,v 1.62 2006/07/20 13:36:28 wwalker Exp $"
+__version__   = "$Revision: 1.62 $"
+__date__      = "$Date: 2006/07/20 13:36:28 $"
+__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__license__   = "LGPL"
+
 import orca.debug as debug
 import orca.default as default
 import orca.atspi as atspi
+import orca.input_event as input_event
 import orca.rolenames as rolenames
 import orca.orca as orca
 import orca.braille as braille
@@ -50,16 +59,19 @@ class Script(default.Script):
         #
         self.debugLevel = debug.LEVEL_FINEST
 
-        # The default for Evolution is to read all table cells (such as in
-        # the mail message header summary list).
-
-        self.readTableCellRow = True
-
         # This will be used to cache a handle to the message area in the
         # Mail compose window.
 
         self.message_panel = None
 
+        # Dictionary of Setup Assistant panels already handled.
+        #
+        self.setupPanels = {}
+
+        # Dictionary of Setup Assistant labels already handled.
+        #
+        self.setupLabels = {}
+
         # The last row and column we were on in the mail message header list.
 
         self.lastMessageColumn = -1
@@ -82,6 +94,105 @@ class Script(default.Script):
                                _("CalendarEvent"),
                                _("calendar event"))
 
+    def setupInputEventHandlers(self):
+        """Defines InputEventHandler fields for this script that can be
+        called by the key and braille bindings. In this particular case,
+        we just want to be able to define our own sayAll() method.
+        """
+
+        default.Script.setupInputEventHandlers(self)
+
+        self.sayAllHandler = input_event.InputEventHandler(
+            Script.sayAll,
+            _("Speaks entire document."))
+
+    def speakSetupAssistantLabel(self, label):
+        """Perform a variety of tests on this Setup Assistant label to see
+        if we want to speak it.
+
+        Arguments:
+        - label: the Setup Assistant Label.
+        """
+
+        if label.state.count(atspi.Accessibility.STATE_SHOWING):
+            # We are only interested in a label if all the panels in the
+            # component hierarchy have states of ENABLED, SHOWING and VISIBLE.
+            # If this is not the case, then just return.
+            #
+            obj = label.parent
+            while obj and obj.role != rolenames.ROLE_APPLICATION:
+                if obj.role == rolenames.ROLE_PANEL:
+                    state = obj.state
+                    if not state.count(atspi.Accessibility.STATE_ENABLED) or \
+                       not state.count(atspi.Accessibility.STATE_SHOWING) or \
+                       not state.count(atspi.Accessibility.STATE_VISIBLE):
+                        return
+                obj = obj.parent
+
+            # Each Setup Assistant screen has one label in the top left
+            # corner that describes what this screen is for. It has a text
+            # weight attribute of 800. We always speak those labels with
+            # " screen" appended.
+            #
+            if label.text:
+                charAttributes = label.text.getAttributes(0)
+                if charAttributes[0]:
+                    charDict = self.textAttrsToDictionary(charAttributes[0])
+                    weight = charDict.get('weight')
+                    if weight and weight == '800':
+                        text = util.getDisplayedText(label)
+
+                        # Only speak the screen label if we haven't already
+                        # done so.
+                        #
+                        if text and not self.setupLabels.has_key(label):
+                            speech.speak(text + _(" screen"), None, False)
+                            self.setupLabels[label] = True
+
+                            # If the locus of focus is a push button that's
+                            # insensitive, speak/braille about it. (The
+                            # Identity screen has such a component).
+                            #
+                            if orca.locusOfFocus and \
+                               orca.locusOfFocus.role == \
+                                   rolenames.ROLE_PUSH_BUTTON and \
+                               (orca.locusOfFocus.state.count( \
+                                   atspi.Accessibility.STATE_SENSITIVE) == 0):
+                                self.updateBraille(orca.locusOfFocus)
+                                speech.speakUtterances(
+                                    self.speechGenerator.getSpeech( \
+                                        orca.locusOfFocus, False))
+
+            # It's possible to get multiple "object:state-changed:showing"
+            # events for the same label. If we've already handled this
+            # label, then just ignore it.
+            #
+            text = util.getDisplayedText(label)
+            if text and not self.setupLabels.has_key(label):
+                # Most of the Setup Assistant screens have a useful piece
+                # of text starting with the word "Please". We want to speak
+                # these. For the first screen, the useful piece of text
+                # starts with "Welcome". For the last screen, it starts
+                # with "Congratulations". Speak those too.
+                #
+                if text.startswith(_("Please")) or \
+                    text.startswith(_("Welcome")) or \
+                    text.startswith(_("Congratulations")):
+                    speech.speak(text, None, False)
+                    self.setupLabels[label] = True
+
+    def handleSetupAssistantPanel(self, panel):
+        """Find all the labels in this Setup Assistant panel and see if
+        we want to speak them.
+
+        Arguments:
+        - panel: the Setup Assistant panel.
+        """
+
+        allLabels = util.findByRole(panel, rolenames.ROLE_LABEL)
+        for i in range(0, len(allLabels)):
+            self.speakSetupAssistantLabel(allLabels[i])
+
     def readPageTab(self, tab):
         """Speak/Braille the given page tab. The speech verbosity is set
            to VERBOSITY_LEVEL_BRIEF for this operation and then restored
@@ -129,6 +240,48 @@ class Script(default.Script):
 
         return hrs + ' ' + mins + ' ' + suffix
 
+    def textLines(self):
+        """This an iterator that produces elements of the form:
+        [SayAllContext, acss], where SayAllContext has the text to be
+        spoken and acss is an ACSS instance for speaking the text.  We
+        do this because of the way Evolution lays out its messages:
+        each paragraph tends to be in its own panel, each of which is
+        in a higher level panel.  So, we just traverse through the
+        children.
+        """
+        panel = orca.locusOfFocus.parent
+        htmlPanel = orca.locusOfFocus.parent.parent
+        startIndex = panel.index
+        for i in range(startIndex, htmlPanel.childCount):
+            accPanel = htmlPanel.accessible.getChildAtIndex(i)
+            panel = atspi.Accessible.makeAccessible(accPanel)
+            accTextObj = panel.accessible.getChildAtIndex(0)
+            textObj = atspi.Accessible.makeAccessible(accTextObj)
+            for [context, acss] in util.textLines(textObj):
+                yield [context, acss]
+
+    def sayAll(self, inputEvent):
+        """Speak all the text associated with the text object that has
+           focus. We have to define our own method here because Evolution
+           does not implement the FLOWS_TO relationship and all the text
+           are in an HTML panel which contains multiple panels, each
+           containing a single text object.
+
+        Arguments:
+        - inputEvent: if not None, the input event that caused this action.
+        """
+
+        debug.println(self.debugLevel, "evolution.sayAll.")
+        if orca.locusOfFocus and orca.locusOfFocus.text:
+            #for [context, acss] in self.textLines():
+            #    print context.utterance
+            speech.sayAll(self.textLines(),
+                          self.__sayAllProgressCallback)
+        else:
+            default.Script.sayAll(self, inputEvent)
+
+        return True
+
     # This method tries to detect and handle the following cases:
     # 1) Mail view: current message pane: individual lines of text.
     # 2) Mail view: current message pane: "standard" mail header lines.
@@ -140,12 +293,15 @@ class Script(default.Script):
     # 8) Mail compose window: message area
     # 9) Spell Checking Dialog
     # 10) Mail view: message area - attachments.
+    # 11) Setup Assistant
 
-    def onFocus(self, event):
-        """Called whenever an object gets focus.
+    def locusOfFocusChanged(self, event, oldLocusOfFocus, newLocusOfFocus):
+        """Called when the visual object with focus changes.
 
         Arguments:
-        - event: the Event
+        - event: if not None, the Event that caused the change
+        - oldLocusOfFocus: Accessible that is the old locus of focus
+        - newLocusOfFocus: Accessible that is the new locus of focus
         """
 
         brailleGen = self.brailleGenerator
@@ -155,7 +311,7 @@ class Script(default.Script):
                                event,
                                event.source.toString())
 
-        # atspi.printAncestry(event.source)
+        # util.printAncestry(event.source)
 
         # 1) Mail view: current message pane: individual lines of text.
         #
@@ -175,13 +331,15 @@ class Script(default.Script):
                      rolenames.ROLE_UNKNOWN]
         if util.isDesiredFocusedItem(event.source, rolesList):
             debug.println(self.debugLevel,
-                      "evolution.onFocus - mail view: current message pane: " \
-                      + "individual lines of text.")
+                          "evolution.locusOfFocusChanged - mail view: " \
+                          + "current message pane: " \
+                          + "individual lines of text.")
 
-            result = atspi.getTextLineAtCaret(event.source)
+            result = util.getTextLineAtCaret(event.source)
             braille.displayMessage(result[0])
+            if settings.enableSpeechIndentation:
+                self.speakTextIndentation(event.source, result[0])
             speech.speak(result[0])
-            orca.setLocusOfFocus(event, event.source, False)
             return
 
         # 2) Mail view: current message pane: "standard" mail header lines.
@@ -207,11 +365,12 @@ class Script(default.Script):
         rolesList = [rolenames.ROLE_TEXT, \
                      rolenames.ROLE_PANEL, \
                      rolenames.ROLE_TABLE_CELL]
-        if self.readTableCellRow \
+        if settings.readTableCellRow \
             and (util.isDesiredFocusedItem(event.source, rolesList)):
             debug.println(self.debugLevel,
-                      "evolution.onFocus - mail view: current message pane: " \
-                      + "standard mail header lines.")
+                          "evolution.locusOfFocusChanged - mail view: " \
+                          + "current message pane: " \
+                          + "standard mail header lines.")
 
             obj = event.source.parent.parent
             parent = obj.parent
@@ -228,12 +387,11 @@ class Script(default.Script):
 
                     if cell.role == rolenames.ROLE_TEXT:
                         regions.append(braille.Text(cell))
-                        result = atspi.getTextLineAtCaret(cell)
+                        result = util.getTextLineAtCaret(cell)
                         utterances.append(result[0])
 
                 braille.displayRegions([regions, regions[0]])
                 speech.speakUtterances(utterances)
-                orca.setLocusOfFocus(event, event.source, False)
                 return
 
         # 3) Mail view: message header list
@@ -257,10 +415,19 @@ class Script(default.Script):
 
         rolesList = [rolenames.ROLE_TABLE_CELL, \
                      rolenames.ROLE_TREE_TABLE]
-        if self.readTableCellRow \
+        if settings.readTableCellRow \
             and (util.isDesiredFocusedItem(event.source, rolesList)):
             debug.println(self.debugLevel,
-                      "evolution.onFocus - mail view: message header list.")
+                          "evolution.locusOfFocusChanged - mail view: " \
+                          + "message header list.")
+
+            # Unfortunately the default read table cell row handling won't
+            # just work with Evolution (see bogusity comment below). We
+            # quickly solve this by setting readTableCellRow to False
+            # for the duration of this code section, then resetting it to
+            # True at the end.
+            #
+            settings.readTableCellRow = False
 
             parent = event.source.parent
             row = parent.table.getRowAtIndex(event.source.index)
@@ -378,11 +545,11 @@ class Script(default.Script):
             if brailleRegions != []:
                 braille.displayRegions([brailleRegions, cellWithFocus])
 
-            orca.setLocusOfFocus(event, event.source, False)
             settings.brailleVerbosityLevel = savedBrailleVerbosityLevel
             settings.speechVerbosityLevel = savedSpeechVerbosityLevel
             self.lastMessageColumn = column
             self.lastMessageRow = row
+            settings.readTableCellRow = True
             return
 
         # 4) Calendar view: day view: tabbing to day with appts.
@@ -408,8 +575,8 @@ class Script(default.Script):
                      rolenames.ROLE_CALENDAR_VIEW]
         if util.isDesiredFocusedItem(event.source, rolesList):
             debug.println(self.debugLevel,
-                      "evolution.onFocus - calendar view: day view: " \
-                      + "tabbing to day with appts.")
+                          "evolution.locusOfFocusChanged - calendar view: " \
+                          + "day view: tabbing to day with appts.")
 
             parent = event.source.parent
             utterances = speechGen.getSpeech(parent, False)
@@ -475,8 +642,8 @@ class Script(default.Script):
                      rolenames.ROLE_CALENDAR_VIEW]
         if util.isDesiredFocusedItem(event.source, rolesList):
             debug.println(self.debugLevel,
-                      "evolution.onFocus - calendar view: day view: " \
-                      + "moving with arrow keys.")
+                      "evolution.locusOfFocusChanged - calendar view: " \
+                      + "day view: moving with arrow keys.")
 
             brailleRegions = []
             index = event.source.index
@@ -552,7 +719,7 @@ class Script(default.Script):
                      rolenames.ROLE_SCROLL_PANE]
         if util.isDesiredFocusedItem(event.source, rolesList):
             debug.println(self.debugLevel,
-                      "evolution.onFocus - preferences dialog: " \
+                      "evolution.locusOfFocusChanged - preferences dialog: " \
                       + "table cell in options list.")
 
             index = event.source.index
@@ -587,8 +754,8 @@ class Script(default.Script):
                      rolenames.ROLE_DIALOG]
         if util.isDesiredFocusedItem(event.source, rolesList):
             debug.println(self.debugLevel,
-                      "evolution.onFocus - mail insert attachment dialog: " \
-                      + "unlabelled button.")
+                          "evolution.locusOfFocusChanged - mail insert " \
+                          + "attachment dialog: unlabelled button.")
 
             brailleRegions = []
             utterance = _("Directories button")
@@ -615,8 +782,8 @@ class Script(default.Script):
                      rolenames.ROLE_SCROLL_PANE]
         if util.isDesiredFocusedItem(event.source, rolesList):
             debug.println(self.debugLevel,
-                      "evolution.onFocus - mail compose window: " \
-                      + "message area.")
+                          "evolution.locusOfFocusChanged - mail " \
+                          + "compose window: message area.")
 
             self.message_panel = event.source.parent.parent
 
@@ -639,11 +806,10 @@ class Script(default.Script):
                     rolenames.ROLE_DIALOG]
         if util.isDesiredFocusedItem(event.source, rolesList):
             debug.println(self.debugLevel,
-                      "evolution.onFocus - spell checking dialog.")
+                      "evolution.locusOfFocusChanged - spell checking dialog.")
 
             # Braille the default action for this component.
             #
-            orca.setLocusOfFocus(event, event.source, False)
             self.updateBraille(orca.locusOfFocus)
 
             # Look for the "Suggestions for 'xxxxx' label in the spell
@@ -651,7 +817,7 @@ class Script(default.Script):
             # misspelt word.
             #
             panel = event.source.parent.parent
-            allLabels = atspi.findByRole(panel, rolenames.ROLE_LABEL)
+            allLabels = util.findByRole(panel, rolenames.ROLE_LABEL)
             found = False
             for i in range(0, len(allLabels)):
                 if not found:
@@ -674,7 +840,7 @@ class Script(default.Script):
             if self.message_panel != None:
                 allTokens = []
                 panel = self.message_panel
-                allText = atspi.findByRole(panel, rolenames.ROLE_TEXT)
+                allText = util.findByRole(panel, rolenames.ROLE_TEXT)
                 for i in range(0, len(allText)):
                     text = allText[i].text.getText(0, -1)
                     tokens = text.split()
@@ -706,26 +872,96 @@ class Script(default.Script):
                     rolenames.ROLE_PANEL]
         if util.isDesiredFocusedItem(event.source, rolesList):
             debug.println(self.debugLevel,
-                      "evolution.onFocus - mail message area attachments.")
+                          "evolution.locusOfFocusChanged - " \
+                          + "mail message area attachments.")
 
             # Speak/braille the default action for this component.
             #
-            default.Script.onFocus(self, event)
+            default.Script.locusOfFocusChanged(self, event,
+                                           oldLocusOfFocus, newLocusOfFocus)
 
             table = event.source.parent.parent.parent.parent.parent
             cell = table.child(table.childCount-1)
-            allText = atspi.findByRole(cell, rolenames.ROLE_TEXT)
+            allText = util.findByRole(cell, rolenames.ROLE_TEXT)
             utterance = "for " + allText[0].text.getText(0, -1)
             speech.speak(utterance)
             return
 
+        # 11) Setup Assistant.
+        #
+        # If the name of the frame of the object that currently has focus is
+        # "Evolution Setup Assistant", then empty out the two dictionaries
+        # containing which setup assistant panels and labels we've already
+        # seen.
+
+        obj = event.source.parent
+        while obj and obj.role != rolenames.ROLE_APPLICATION:
+            if obj.role == rolenames.ROLE_FRAME and \
+                obj.name.endswith(_("Assistant")):
+                debug.println(self.debugLevel,
+                              "evolution.locusOfFocusChanged - " \
+                              + "setup assistant.")
+                self.setupPanels = {}
+                self.setupLabels = {}
+                break
+            obj = obj.parent
+
         # For everything else, pass the focus event onto the parent class
         # to be handled in the default way.
         #
         # Note that this includes table cells if we only want to read the
         # current cell.
 
-        default.Script.onFocus(self, event)
+        default.Script.locusOfFocusChanged(self, event,
+                                           oldLocusOfFocus, newLocusOfFocus)
+
+    def onStateChanged(self, event):
+        """Called whenever an object's state changes.  We are only
+        interested in "object:state-changed:showing" events for any
+        object in the Setup Assistant.
+
+        Arguments:
+        - event: the Event
+        """
+
+        if event.type.endswith("showing"):
+            # Check to see if this "object:state-changed:showing" event is
+            # for an object in the Setup Assistant by walking back up the
+            # object hierarchy until we get to the frame object and check
+            # to see if it has a name that ends with "Assistant", which is
+            # what we see when we configure Evolution for the first time
+            # and when we add new accounts.
+            #
+            obj = event.source.parent
+            while obj and obj.role != rolenames.ROLE_APPLICATION:
+                if obj.role == rolenames.ROLE_FRAME and \
+                    obj.name.endswith(_("Assistant")):
+                    debug.println(self.debugLevel,
+                                  "evolution.onStateChanged - " \
+                                  + "setup assistant.")
+
+                    # If the event is for a label see if we want to speak it.
+                    #
+                    if event.source.role == rolenames.ROLE_LABEL:
+                        self.speakSetupAssistantLabel(event.source)
+                        break
+
+                    # If the event is for a panel and we haven't already
+                    # seen this panel, then handle it.
+                    #
+                    elif event.source.role == rolenames.ROLE_PANEL and \
+                        not self.setupPanels.has_key(event.source):
+                        self.handleSetupAssistantPanel(event.source)
+                        self.setupPanels[event.source] = True
+                        break
+
+                obj = obj.parent
+
+        # For everything else, pass the event onto the parent class
+        # to be handled in the default way.
+        #
+        default.Script.onStateChanged(self, event)
+
 
 # Values used to construct a time string for calendar appointments.
 #
