@@ -17,17 +17,13 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
-"""Custom script for gedit.  [[[TODO: WDW - HACK because tickling gedit
-when it is starting can cause gedit to issue the following message:
+"""Custom script for gedit."""
 
-     (gedit:31434): GLib-GObject-WARNING **: invalid cast from `SpiAccessible' to `BonoboControlAccessible'
-
-It seems as though whenever this message is issued, gedit will hang when
-you try to exit it.  Debugging has shown that the iconfied state in
-particular seems to indicate that an object is telling all assistive
-technologies to just leave it alone or it will pull the trigger on the
-application.]]]
-"""
+__id__        = "$Id: gedit.py,v 1.15 2006/06/10 00:54:48 wwalker Exp $"
+__version__   = "$Revision: 1.15 $"
+__date__      = "$Date: 2006/06/10 00:54:48 $"
+__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__license__   = "LGPL"
 
 import orca.debug as debug
 import orca.atspi as atspi
@@ -89,6 +85,10 @@ class Script(default.Script):
 
         default.Script.__init__(self, app)
 
+        # Set the debug level for all the methods in this script.
+        #
+        self.debugLevel = debug.LEVEL_FINEST
+
         # This will be used to cache a handle to the gedit text area for
         # spell checking purposes.
 
@@ -123,7 +123,6 @@ class Script(default.Script):
 
         # Braille the default action for this component.
         #
-        orca.setLocusOfFocus(event, event.source, False)
         self.updateBraille(orca.locusOfFocus)
 
         # Look for the label containing the misspelled word.
@@ -131,7 +130,7 @@ class Script(default.Script):
         # Spelling dialog. Look for the one that isn't a label to
         # another component.
         #
-        allLabels = atspi.findByRole(panel, rolenames.ROLE_LABEL)
+        allLabels = util.findByRole(panel, rolenames.ROLE_LABEL)
         for i in range(0, len(allLabels)):
             if allLabels[i].name.startswith(_("Change to:")) or \
                allLabels[i].name.startswith(_("Misspelled word:")):
@@ -147,10 +146,10 @@ class Script(default.Script):
         # was called. If they are the same then we ignore it.
 
         if self.textArea != None:
-            allText = atspi.findByRole(self.textArea, rolenames.ROLE_TEXT)
+            allText = util.findByRole(self.textArea, rolenames.ROLE_TEXT)
             caretPosition = allText[0].text.caretOffset
 
-            debug.println(debug.LEVEL_FINEST, \
+            debug.println(self.debugLevel, \
                 "gedit.readMisspeltWord: type=%s  word=%s caret position=%d" \
                 % (event.type, badWord, caretPosition))
 
@@ -193,21 +192,23 @@ class Script(default.Script):
     # 1) Text area (for caching handle for spell checking purposes).
     # 2) Check Spelling Dialog.
 
-    def onFocus(self, event):
-        """Called whenever an object gets focus.
+    def locusOfFocusChanged(self, event, oldLocusOfFocus, newLocusOfFocus):
+        """Called when the visual object with focus changes.
 
         Arguments:
-        - event: the Event
+        - event: if not None, the Event that caused the change
+        - oldLocusOfFocus: Accessible that is the old locus of focus
+        - newLocusOfFocus: Accessible that is the new locus of focus
         """
 
         brailleGen = self.brailleGenerator
         speechGen = self.speechGenerator
 
-        debug.printObjectEvent(debug.LEVEL_FINEST,
+        debug.printObjectEvent(self.debugLevel,
                                event,
                                event.source.toString())
 
-        # atspi.printAncestry(event.source)
+        # util.printAncestry(event.source)
 
         # 1) Text area (for caching handle for spell checking purposes).
         #
@@ -226,8 +227,8 @@ class Script(default.Script):
                      rolenames.ROLE_PAGE_TAB_LIST,
                      rolenames.ROLE_SPLIT_PANE]
         if util.isDesiredFocusedItem(event.source, rolesList):
-            debug.println(debug.LEVEL_FINEST,
-                      "gedit.onFocus - text area.")
+            debug.println(self.debugLevel,
+                          "gedit.locusOfFocusChanged - text area.")
 
             self.textArea = event.source.parent
             # Fall-thru to process the event with the default handler.
@@ -252,8 +253,8 @@ class Script(default.Script):
         if util.isDesiredFocusedItem(event.source, rolesList):
             frame = event.source.parent.parent.parent.parent
             if frame.name.startswith(_("Check Spelling")):
-                debug.println(debug.LEVEL_FINEST,
-                      "gedit.onFocus - check spelling dialog.")
+                debug.println(self.debugLevel,
+                        "gedit.locusOfFocusChanged - check spelling dialog.")
 
                 self.readMisspeltWord(event, event.source.parent.parent)
                 # Fall-thru to process the event with the default handler.
@@ -261,7 +262,8 @@ class Script(default.Script):
         # For everything else, pass the focus event onto the parent class
         # to be handled in the default way.
 
-        default.Script.onFocus(self, event)
+        default.Script.locusOfFocusChanged(self, event,
+                                           oldLocusOfFocus, newLocusOfFocus)
 
     # This method tries to detect and handle the following cases:
     # 1) check spelling dialog.
@@ -276,11 +278,11 @@ class Script(default.Script):
         brailleGen = self.brailleGenerator
         speechGen = self.speechGenerator
 
-        debug.printObjectEvent(debug.LEVEL_FINEST,
+        debug.printObjectEvent(self.debugLevel,
                                event,
                                event.source.toString())
 
-        # atspi.printAncestry(event.source)
+        # util.printAncestry(event.source)
 
         # 1) check spelling dialog.
         #
@@ -304,7 +306,7 @@ class Script(default.Script):
         if util.isDesiredFocusedItem(event.source, rolesList):
             frame = event.source.parent.parent.parent
             if frame.name.startswith(_("Check Spelling")):
-                debug.println(debug.LEVEL_FINEST,
+                debug.println(self.debugLevel,
                       "gedit.onNameChanged - check spelling dialog.")
 
                 self.readMisspeltWord(event, event.source.parent)
