@@ -21,9 +21,9 @@
 of manipulating text - both the user and the system put text into the
 system and we try to determine which was which and why."""
 
-__id__        = "$Id: gnome-terminal.py,v 1.10 2006/06/10 00:54:48 wwalker Exp $"
-__version__   = "$Revision: 1.10 $"
-__date__      = "$Date: 2006/06/10 00:54:48 $"
+__id__        = "$Id: gnome-terminal.py,v 1.12 2006/06/27 15:55:13 richb Exp $"
+__version__   = "$Revision: 1.12 $"
+__date__      = "$Date: 2006/06/27 15:55:13 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -75,7 +75,8 @@ class Script(default.Script):
         - event: the Event
         """
 
-        if orca.lastInputEvent:
+        if orca.lastInputEvent \
+               and isinstance(orca.lastInputEvent, input_event.KeyboardEvent):
             event_string = orca.lastInputEvent.event_string
         else:
             event_string = None
@@ -139,6 +140,11 @@ class Script(default.Script):
         #
         # We'll let our super class handle "Delete".  We'll handle Ctrl+D.
         #
+
+        if not orca.lastInputEvent or \
+            not isinstance(orca.lastInputEvent, input_event.KeyboardEvent):
+            return
+
         keyString = orca.lastInputEvent.event_string
 
         controlPressed = orca.lastInputEvent.modifiers \
