@@ -19,9 +19,9 @@
 
 """Provides various utility functions for Orca."""
 
-__id__        = "$Id: util.py,v 1.21 2006/06/19 18:54:00 richb Exp $"
-__version__   = "$Revision: 1.21 $"
-__date__      = "$Date: 2006/06/19 18:54:00 $"
+__id__        = "$Id: util.py,v 1.27 2006/07/21 14:58:27 wwalker Exp $"
+__version__   = "$Revision: 1.27 $"
+__date__      = "$Date: 2006/07/21 14:58:27 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -39,8 +39,10 @@ import string
 
 import atspi
 import debug
+import input_event
 import orca
 import rolenames
+import settings
 import speech
 import speechserver
 
@@ -50,14 +52,14 @@ def appendString(text, newText, delimiter=" "):
     """Appends the newText to the given text with the delimiter in between
     and returns the new string.  Edge cases, such as no initial text or
     no newText, are handled gracefully."""
-    
+
     if (not newText) or (len(newText) == 0):
         return text
     elif text and len(text):
         return text + delimiter + newText
     else:
         return newText
-        
+
 def getDisplayedLabel(object):
     """If there is an object labelling the given object, return the
     text being displayed for the object labelling this object.
@@ -69,18 +71,18 @@ def getDisplayedLabel(object):
     Returns the string of the object labelling this object, or None
     if there is nothing of interest here.
     """
-    label = None
-    relations = object.relations
-    
+
     # For some reason, some objects are labelled by the same thing
     # more than once.  Go figure, but we need to check for this.
     #
+    label = None
+    relations = object.relations
     allTargets = []
-    
+
     for relation in relations:
         if relation.getRelationType() \
                == atspi.Accessibility.RELATION_LABELLED_BY:
-            
+
             # The object can be labelled by more than one thing, so we just
             # get all the labels (from unique objects) and append them
             # together.  An example of such objects live in the "Basic"
@@ -93,10 +95,71 @@ def getDisplayedLabel(object):
                 if not target in allTargets:
                     allTargets.append(target)
                     label = appendString(label, getDisplayedText(target))
+
+    # [[[TODO: HACK - we've discovered oddness in hierarchies such as
+    # the gedit Edit->Preferences dialog.  In this dialog, we have
+    # labeled groupings of objects.  The grouping is done via a FILLER
+    # with two children - one child is the overall label, and the
+    # other is the container for the grouped objects.  When we detect
+    # this, we add the label to the overall context.
+    #
+    # We are also looking for PANELs with two children, because we
+    # discovered similer cases for PANELS in java applications.  The
+    # java application that includes this behavior is FileChooserDemo.
+    # Choose Show FileChooser button, a new dialog appears. In this
+    # dialog, the issue presented is for "Files of Type" combo
+    # box. Moving to this combo, Orca speaks only the selected item
+    # from combo, "Files of Type" label is not spoken.]]]
+    #
+    if (not label) \
+        and ((object.role == rolenames.ROLE_FILLER) \
+             or (object.role == rolenames.ROLE_PANEL)) \
+        and (object.childCount == 2):
+
+        potentialLabel = object.child(0)
+        labeledObject = object.child(1)
+
+        if potentialLabel and labeledObject \
+            and (potentialLabel.role == rolenames.ROLE_LABEL):
+
+            # If the label is labeling something, we will ignore it
+            # because we're likely to pick it up via some other means.
+            # An example here is the "Shade transparent or image background:"
+            # label in gnome-terminal's "Effects" customization tab.
+            #
+            useLabel = True
+            for relation in potentialLabel.relations:
+                if relation.getRelationType() \
+                    == atspi.Accessibility.RELATION_LABEL_FOR:
+
+                    # If potentialLabel is labeling labeledObject, we
+                    # check to see if the labeledObject has the
+                    # LABELLED_BY relation.  If the labeledObject
+                    # doensn't have a LABELLED_BY relation, then the
+                    # potentialLabel is likely not to be picked up via
+                    # other means, so we use this as a label for the
+                    # FILLER or PANEL.
+                    #
+                    useLabel = False
+                    for i in range(0, relation.getNTargets()):
+                        target = atspi.Accessible.makeAccessible(
+                            relation.getTarget(i))
+                        if target == labeledObject:
+                            useLabel = True
+                            for rel in labeledObject.relations:
+                                if rel.getRelationType() \
+                                   == atspi.Accessibility.RELATION_LABELLED_BY:
+                                    useLabel = False
+                                    break
+                        break
+                    break
+            if useLabel:
+                label = potentialLabel.name
+
     return label
 
 def __getDisplayedTextInComboBox(combo):
-    
+
     """Returns the text being displayed in a combo box.  If nothing is
     displayed, then None is returned.
 
@@ -108,7 +171,7 @@ def __getDisplayedTextInComboBox(combo):
     """
 
     displayedText = None
-    
+
     # Find the text displayed in the combo box.  This is either:
     #
     # 1) The last text object that's a child of the combo box
@@ -192,7 +255,7 @@ def getDisplayedText(obj):
                 childText = getDisplayedText(child)
                 if childText and len(childText):
                     displayedText = appendString(displayedText, childText)
- 
+
     return displayedText
 
 def getRealActiveDescendant(obj):
@@ -254,6 +317,28 @@ def findFocusedObject(root):
 
     return None
 
+def isDoubleClick(lastInputEvent, inputEvent):
+    """Return an indication of whether the user has double-clicked a
+    one of the keys on the keyboard.
+
+    Arguments:
+    - lastInputEvent: the last input event.
+    - inputEvent: the current input event.
+    """
+
+    if not isinstance(lastInputEvent, input_event.KeyboardEvent) or \
+       not isinstance(inputEvent, input_event.KeyboardEvent):
+        return False
+
+    if (lastInputEvent.hw_code != inputEvent.hw_code) or \
+       (lastInputEvent.modifiers != inputEvent.modifiers):
+        return False
+
+    if (inputEvent.time - lastInputEvent.time) < settings.doubleClickTimeout:
+        return True
+    else:
+        return False
+
 def isDesiredFocusedItem(obj, rolesList):
     """Called to determine if the given object and it's hierarchy of
        parent objects, each have the desired roles.
@@ -364,7 +449,7 @@ def textLines(obj):
             #if string[-1:] == "\n":
             #    string = string[0][:-1]
 
-            yield [speechserver.SayAllContext(obj, string, 
+            yield [speechserver.SayAllContext(obj, string,
                                               startOffset, endOffset),
                    None]
 
@@ -598,7 +683,7 @@ def getObjects(root, onlyShowing=True):
     Arguments:
     - root:        the Accessible object to traverse
     - onlyShowing: examine only those objects that are SHOWING
-    
+
     Returns: a list of all objects under the specified object
     """
 
@@ -919,7 +1004,7 @@ def outlineAccessible(accessible, erasePrevious=True):
 
 def isTextSelected(obj, startOffset, endOffset):
     """Returns an indication of whether the text is selected by
-    comparing the text offset with the various selected regions of 
+    comparing the text offset with the various selected regions of
     text for this accessible object.
 
     Arguments:
@@ -1041,7 +1126,7 @@ def speakTextSelectionState(obj, startOffset, endOffset):
                     speech.speak(_("unselected"), None, False)
                     break
 
-    # Save away the current text cursor position and list of text 
+    # Save away the current text cursor position and list of text
     # selections for next time.
     #
     obj.lastCursorPosition = obj.text.caretOffset
